﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.IO;
using System.Reflection;
using PuzzlePlayer_Namespace.Properties;    // for accessing user settings

namespace PuzzlePlayer_Namespace
{
    // struct for saving themes
    internal struct Theme(string s, Color col1, Color col2, Color col3, bool isUnlockedByDefault)
    {
        public const string prefix = "THEME";
        public string name = s;
        public Color primaryColor = col1;
        public Color secondaryColor = col2;
        public Color tertiaryColor = col3;
        public bool unlocked = isUnlockedByDefault; // keeps track if this theme is unlocked

        public override string ToString()
        {
            return $"{prefix}|{name}|{primaryColor.R}|{primaryColor.G}|{primaryColor.B}|{secondaryColor.R}|{secondaryColor.G}|{secondaryColor.B}|{tertiaryColor.R}|{tertiaryColor.G}|{tertiaryColor.B}|{unlocked}";
        }

        public static Theme? FromString(string s)
        {
            string[] split = s.Split('|');
            if (split[0] != prefix)
                return null;
            try
            {
                Color c1 = Color.FromArgb(int.Parse(split[2]),int.Parse(split[3]),int.Parse(split[4]));
                Color c2 = Color.FromArgb(int.Parse(split[5]),int.Parse(split[6]),int.Parse(split[7]));
                Color c3 = Color.FromArgb(int.Parse(split[8]),int.Parse(split[9]),int.Parse(split[10]));

                return new Theme(split[1], c1, c2, c3, bool.Parse(split[11]));
            }
            catch (Exception)
            {
                return null;
            }
        }

        public void Unlock()
        {
            unlocked = true;
        }
    }

    // Class for managing and storing user data
    internal class UserDataManager
    {
        // properties for getting various settings
        public int Money { get { return Settings.Default.Money; } set { SetMoney(value); } }
        public Font MainFont { get { return GetMainFont(); } }
        public Theme Theme { get { return GetCurrentTheme(); } }
        
        private List<Theme> allThemes;

        private const string themeFilePath = "PuzzlePlayer_Namespace.Resources.Themes.txt";

        public UserDataManager()
        {
            allThemes = GetThemes();
        }

        private List<Theme> GetThemes()
        {
            List<Theme> result = new List<Theme>();

            using(Stream stream = Assembly.GetExecutingAssembly().GetManifestResourceStream(themeFilePath))
            {
                using(StreamReader sr = new StreamReader(stream))
                {
                    while (!sr.EndOfStream)
                    {
                        Theme? t = Theme.FromString(sr.ReadLine());
                        if (t == null)
                            throw new Exception("blijf van de theme file af kkr djalla, ga t zelf maar fixen");
                        else
                            result.Add((Theme)t);
                    }
                }
            }
            return result;
        }

        private void SetMoney(int newValue)
        {
            if (newValue < 0)
                return;

            Settings.Default.Money = newValue;
            Settings.Default.Save();
        }

        private Font GetMainFont()
        {
            string fontName = Settings.Default.FontName;
            int fontSize = Settings.Default.FontSize;

            return new Font(fontName, fontSize);
        }

        public void SetMainFont(string fontName, int fontSize)
        {
            Settings.Default.FontName = fontName;
            Settings.Default.FontSize = fontSize;
            Settings.Default.Save();
        }

        private Theme GetCurrentTheme()
        {
            string currentThemeName = Settings.Default.ThemeName;
            foreach (Theme t in allThemes)
            {
                if (t.unlocked && currentThemeName == t.name)
                    return t;
            }
            throw new Exception("can't find the selected theme in unlocked themes");
        }

        public void SetCurrentTheme(string newThemeName)
        {
            foreach (Theme t in allThemes)
            {
                if(t.unlocked && newThemeName == t.name)
                {
                    Settings.Default.ThemeName = newThemeName;
                    Settings.Default.Save();
                    return;
                }
            }
        }

        public void UnlockTheme(string ThemeToUnlock)
        {
            // update the unlockedThemes list
            for (int i = 0; i < allThemes.Count; i++)
            {
                if (allThemes[i].name == ThemeToUnlock)
                {
                    allThemes[i].Unlock();
                    return;
                }
            }

            // write the data back to the file
            using (Stream stream = Assembly.GetExecutingAssembly().GetManifestResourceStream(themeFilePath))
                using (StreamWriter writer = new StreamWriter(stream))
                    foreach(Theme t in allThemes)
                        writer.WriteLine(t.ToString());
        }
    }
}
